#!/usr/bin/env python
#Tiago de Freitas Pereira <tiagofrepereira@gmail.com>
#Sat Oct 20 7:51:00 CET 2012

"""Support methods to compute the DoG from the image
"""

import os, sys
import numpy
import bob


def normalizeFace(frame,bbx,normfacesize,facesize_filter):
  """
  Crop a face and normalize a face

  frame
    Gray frame 

  bbx
    Locations

  normfacesize
    Normalized face

  facesize_filter
    Considers as invalid all the bounding boxes with size smaller then this value
  """

  if(not bbx.is_valid(faceSizeFilter=facesize_filter)):
    return numpy.ones((normfacesize,normfacesize),dtype='uint8')*numpy.nan


  cutframe = frame[bbx.y:(bbx.y+bbx.height),bbx.x:(bbx.x+bbx.width)] # cutting the box region
  tempbbx = numpy.ndarray((normfacesize, normfacesize), 'float64')
  bob.ip.scale(cutframe, tempbbx) # normalization
  tempbbx_ = tempbbx + 0.5
  tempbbx_ = numpy.floor(tempbbx_)
  faceFrame = numpy.cast['uint8'](tempbbx_)
  
  return faceFrame


def calcDoG(normFrame,s1,s2,radius):
  """
  Apply the DoG filter to an specific grayscale frame.

  normFrame
    Normalized face

  location
    The face location

  s1
    Sigma for the first Gaussian

  s2
    Sigma for the second Gaussian

  radius
    The radius for the both gaussians


  """
  #If there is some nan, do nothing
  if(numpy.isnan(sum(sum(normFrame)))):
    return normFrame

  filter1 = bob.ip.Gaussian(radius_x=radius,radius_y=radius,sigma_y=s1,sigma_x=s1)
  filter2 = bob.ip.Gaussian(radius_x=radius,radius_y=radius,sigma_y=s2,sigma_x=s2)

  filteredImage1 = filter1(normFrame)
  filteredImage2 = filter2(normFrame)

  return filteredImage2 - filteredImage1 
  

def create_full_dataset(files,inputDir,retrieveNanLines=False):
  """
  Return a numpy array with all DoG features
 
  files 
    List of file objects with the data
 
  inputDir
    The data dir

  retrieveNanLines
    Retrieve the nan Lines
  """
  dataset = None

  for obj in files:

    filename = str(obj.make_path(inputDir,extension='.hdf5'))
    data = bob.io.load(filename)

    if dataset is None:
      dataset = data
    else:
      dataset = numpy.concatenate((dataset, data),axis=0)


    #Will remove the Nan data 
    if(not retrieveNanLines):
      #selecting the lines with nan
      nanLines = numpy.array([numpy.sum(numpy.isnan(dataset[j,:])) for j in range(dataset.shape[0])])
      nanLines = numpy.where(nanLines>0)[0]

        #removing the lines with nan
      dataset = numpy.delete(dataset,nanLines,axis=0)

  return dataset

